# fx-92 Scientifique Collège+ language interpreter: Input key sequences
#
# This module compiles programs to sequences of key presses that can be used to
# input them on the fx-92 SC+. This is used for Aditya02's hardware mod of the
# fx-92 SC+ which can automatically perform these presses:
#   <https://www.planet-casio.com/Fr/forums/topic16979-1-une-memoire-externe-pour-la-casio-fx92.html>

from fx92.ast import N, Node

def print_key_sequence(n, indent=0):
    if isinstance(n, Node) and n.type == N.PROGRAM:
        for arg in n.args:
            print_key_sequence(arg, indent)
        return

    def p(*args):
        for arg in args:
            print(" "*indent + arg)

    var_numbering = {
        "A": "1",   "B": "2",   "C": "3",   "D": "4",
        "E": "5",   "F": "6",   "M": "7"
    }

    if not isinstance(n, Node):
        print("[Not a node?] {}: {}".format(n, type (n)))
        return

    if n.type == N.CONST:
        p("# CONST {}".format(n.args[0]))
        p("<TODO>")

    elif n.type == N.VAR:
        p("# VAR {}".format(n.args[0]))
        p("<TODO>")

    elif n.type == N.REL:
        p("# REL {}".format(n.args[0]))
        p("<TODO>")

    elif n.type == N.FUN:
        p("# FUN {}".format(n.args[0]))
        p(n.args[0] + "()")
        for arg in n.args[1]:
            print_key_sequence(arg, indent+1)
        return

    # OPTN, first page

    elif n.type == N.FORWARD:
        p("# FORWARD {}".format(n.args[0]))
        p("OPTN", "1")
        print_key_sequence(n.args[0], indent=indent+2)
        p("EXE", "EXE")

    elif n.type == N.ROTATE:
        p("# ROTATE {}".format(n.args[0]))
        p("OPTN", "2")
        print_key_sequence(n.args[0], indent=indent+2)
        p("EXE", "EXE")

    elif n.type == N.ORIENT:
        p("# ORIENT {}".format(n.args[0]))
        p("OPTN", "3")
        print_key_sequence(n.args[0], indent=indent+2)
        p("EXE", "EXE")

    elif n.type == N.GOTO:
        p("# GOTO {} {}".format(n.args[0], n.args[1]))
        p("OPTN", "4")
        print_key_sequence(n.args[0], indent=indent+2)
        p("EXE")
        print_key_sequence(n.args[1], indent=indent+2)
        p("EXE", "EXE")

    # OPTN, second page

    elif n.type == N.PENDOWN:
        p("# PENDOWN")
        p("OPTN", "DOWN", "1")

    elif n.type == N.PENUP:
        p("# PENUP")
        p("OPTN", "DOWN", "2")

    elif n.type == N.ASSIGN:
        p("# ASSIGN {} {}".format(n.args[0], n.args[1]))
        p("OPTN", "DOWN", "3")
        print_key_sequence(n.args[0], indent=indent+2)
        p("EXE")
        if n.args[1] != "A":
            p("RIGHT", var_numbering[n.args[1]])
        p("EXE")

    else:
        print("[???]", n.type, n.type.name.lower())
#

__all__ = ["print_key_sequence"]
